# @summary Install and configure Sympa mailing list manager
#
# Install and configure Sympa mailing list manager
#
# @param manage_user if true this puppet module manage user account settings.
# @param user User account name
# @param group User account group name.
# @param manage_database If true this module manage postgreSQL server and database backend.
# @param db_host Address of the database server with TCP connection (5432 is used as port).
# @param db_name Name of used database.
# @param db_user User for the database connection.
# @param db_password Password for the database connection.
# @param prefix_path Install Sympa files in this path.
# @param sysconf_dir Configuration dir under prefix_path
# @param aliases_file Aliases file to store Sympa mail aliases, under sysconfig_dir
# @param release Installed release number of Sympa.
# @param listdomain
#   Primary mail domain name. Web interface for Sympa mailing list manager, can
#   be joined at https://$listdomain
# @param listmaster Email addresses of listmaster
# @param lang Supported language
# @param topics Defines topics (categories) of the mailing lists.
#
# @example
#   include sympa
class sympa (
  Boolean $manage_user = true,
  String[1] $user = 'sympa',
  String[1] $group = 'sympa',
  Boolean $manage_database = true,
  String[1] $db_host = 'localhost',
  String[1] $db_name = 'sympa',
  String[1] $db_user = 'sympa',
  Variant[String[1], Sensitive[String[1]]] $db_password = 'changeme',
  Stdlib::Absolutepath $prefix_path = '/home/sympa',
  String[1] $sysconf_dir = 'etc',
  String[1] $aliases_file = 'sympa_aliases',
  String[1] $release = '6.2.74',
  Stdlib::Fqdn $listdomain = 'listes.example.org',
  Stdlib::Email $listmaster = 'listmaster@example.org',
  Enum['fr'] $lang = 'fr',
  Sympa::Topics $topics = {},
) {
  $_required = [
    'debconf-utils',
    'cpanminus',
    'gcc',
    'libssl-dev',
    'autoconf',
    'automake',
    'gettext',
    'mhonarc',
    'perl',
    'libxml-libxml-perl',
    'libmime-lite-html-perl',
    'libdatetime-timezone-perl',
  ]

  $_configure_array = [
    '--enable-fhs',
    "--prefix=${sympa::prefix_path}",
    "--sysconfdir=${sympa::prefix_path}/${sympa::sysconf_dir}",
    "--with-confdir=${sympa::prefix_path}/${sympa::sysconf_dir}",
    "--with-user=${sympa::user}",
    "--with-group=${sympa::group}",
    "--with-aliases_file=${sympa::prefix_path}/${sympa::sysconf_dir}/${sympa::aliases_file}",
    '--without-initdir',
  ]

  $_configure_opts = join($_configure_array, ' ')

  if $sympa::manage_user {
    class { 'accounts':
      user_list => {
        $sympa::user => {
          comment    => 'Sympa Mailing List',
          group      => $sympa::group,
          shell      => '/usr/sbin/nologin',
          managehome => true,
          home       => $sympa::prefix_path,
          home_mode  => '0701',
        },
      },
    }

    Class['accounts'] -> File["${sympa::prefix_path}/sympa"]
    Class['accounts'] -> Exec['autoreconf']
    Class['accounts'] -> Exec['configure']
    Class['accounts'] -> Exec['make']
  }
  if $sympa::manage_database {
    include postgresql::server

    postgresql::server::db { $sympa::db_name:
      user     => $sympa::db_user,
      password => postgresql::postgresql_password($sympa::db_user, $sympa::db_password),
    }

    package { 'libdbd-pg-perl':
      ensure  => present,
      require => Class['Postgresql::Server'],
    }

    Postgresql::Server::Db[$sympa::db_name] -> File["${sympa::prefix_path}/${sympa::sysconf_dir}/sympa.conf"]
  }

  archive { "sympa-${sympa::release}.tar.gz":
    path         => "/tmp/sympa-${sympa::release}.tar.gz",
    source       => "https://github.com/sympa-community/sympa/archive/refs/tags/${sympa::release}.tar.gz",
    user         => $sympa::user,
    extract      => true,
    extract_path => $sympa::prefix_path,
    creates      => "${sympa::prefix_path}/sympa-${sympa::release}",
    cleanup      => true,
  }
  -> file { "${sympa::prefix_path}/sympa":
    ensure => link,
    owner  => $user,
    target => "${sympa::prefix_path}/sympa-${sympa::release}",
  }
  -> package { $_required:
    ensure => present,
  }
  -> exec { 'cpanm --installdeps':
    command     => '/usr/bin/cpanm --installdeps --with-recommends .',
    cwd         => "${sympa::prefix_path}/sympa-${sympa::release}",
    environment => ['HOME=/root'],
    user        => 'root',
    timeout     => 1200,
    creates     => [
      '/usr/local/share/perl/5.34.0/Mail/DKIM/Algorithm/Base.pm',
      '/usr/local/share/perl/5.34.0/Crypt/OpenSSL/Guess.pm',
      '/usr/local/share/perl/5.34.0/Net/DNS/Resolver/Base.pm',
    ],
    subscribe   => [
      Archive["sympa-${sympa::release}.tar.gz"],
      File["${sympa::prefix_path}/sympa"],
    ],
  }
  -> exec { 'autoreconf':
    command     => '/usr/bin/autoreconf -i',
    cwd         => "${sympa::prefix_path}/sympa-${sympa::release}",
    environment => ["HOME=${sympa::prefix_path}"],
    user        => $sympa::user,
    creates     => "${sympa::prefix_path}/sympa-${sympa::release}/configure",
  }
  -> exec { 'configure':
    command     => "/usr/bin/make distclean; ${sympa::prefix_path}/sympa/configure ${_configure_opts}",
    path        => '/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin',
    cwd         => "${sympa::prefix_path}/sympa-${sympa::release}",
    environment => ["HOME=${sympa::prefix_path}"],
    user        => $sympa::user,
    creates     => "${sympa::prefix_path}/sympa-${sympa::release}/config.status",
  }
  ~> exec { 'make':
    command     => '/usr/bin/make',
    cwd         => "${sympa::prefix_path}/sympa-${sympa::release}",
    environment => ["HOME=${sympa::prefix_path}"],
    user        => $sympa::user,
    creates     => "${sympa::prefix_path}/sympa-${sympa::release}/service/logrotate",
  }
  ~> exec { 'make install':
    command     => '/usr/bin/make install',
    cwd         => "${sympa::prefix_path}/sympa-${sympa::release}",
    environment => ["HOME=${sympa::prefix_path}"],
    user        => $sympa::user,
    creates     => [
      "${sympa::prefix_path}/etc",
      "${sympa::prefix_path}/var",
      "${sympa::prefix_path}/sbin",
    ],
  }
  -> exec { 'locale-gen fr_FR.UTF-8':
    command => 'locale-gen fr_FR.UTF-8',
    path    => '/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin',
    user    => 'root',
    unless  => 'debconf-get-selections | grep -qP "^locales\s+locales/locales_to_be_generated\s+multiselect\s+.*fr_FR.UTF-8 UTF-8"',
  }
  -> exec { 'debconf-set-selections locales locales/locales_to_be_generated':
    command => 'echo "locales locales/locales_to_be_generated multiselect en_US.UTF-8 UTF-8, fr_FR.UTF-8 UTF-8" | debconf-set-selections',
    path    => '/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin',
    user    => 'root',
    unless  => 'debconf-get-selections | grep -qP "^locales\s+locales/locales_to_be_generated\s+multiselect\s+.*fr_FR.UTF-8 UTF-8"',
  }
  -> file { "${sympa::prefix_path}/${sympa::sysconf_dir}/sympa.conf":
    ensure  => file,
    mode    => '0600',
    owner   => $sympa::user,
    group   => $sympa::group,
    content => epp('sympa/sympa.conf.epp'),
    notify  => [
      Service['sympa.service'],
      Service['wwsympa.service'],
    ],
  }

  if $topics.empty {
    # topics are defaults
    file { "${sympa::prefix_path}/${sympa::sysconf_dir}/topics.conf":
      ensure => 'absent',
    }
  } else {
    file { "${sympa::prefix_path}/${sympa::sysconf_dir}/topics.conf":
      ensure  => file,
      mode    => '0755',
      owner   => $sympa::user,
      group   => $sympa::group,
      content => epp('sympa/topics.conf.epp'),
    }
  }

  [
    'sympa.service',
    'sympa-archive.service',
    'sympa-bounce.service',
    'sympa-outgoing.service',
    'sympa-task.service',
    'wwsympa.socket',
    'wwsympa.service',
  ].each | String $_unit | {
    systemd::unit_file { $_unit:
      enable  => true,
      active  => true,
      content => epp("sympa/${_unit}.epp"),
      require => [
        File["${sympa::prefix_path}/${sympa::sysconf_dir}/sympa.conf"],
        Exec['locale-gen fr_FR.UTF-8'],
      ],
    }
  }
}
